/*
 *  GNOME IrDA Monitor (girda)
 *  Copyright (C) 2001 Ragnar Henriksen <ragnar@juniks.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <glade/glade.h>
//#include <gnome.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include <panel-applet.h>
#include <panel-applet-gconf.h>

#include "girda_app.h"
#include "girda_cb.h"
#include "girda_applet.h"


gint girda_load_gfx( Irda_applet_t *wdata );

Irda_applet_t *GIrDAdata;

static const BonoboUIVerb GIrDA_Applet_menu_verbs [] = {
        BONOBO_UI_UNSAFE_VERB ("GIrDA_Properties", prop_cb),
        BONOBO_UI_UNSAFE_VERB ("GIrDA_Help",       help_cb),
        BONOBO_UI_UNSAFE_VERB ("GIrDA_About",      about_cb),
        BONOBO_UI_VERB_END
};


GdkPixbuf *
girda_load_pixbuf( const char *file )
{
	char *filelocation;
	GError *error = NULL;
	GdkPixbuf *pixbuf;

	filelocation = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, file, FALSE, NULL);
	pixbuf = gdk_pixbuf_new_from_file (filelocation, &error);
	g_free (filelocation);
   
	if (error) {
		g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
		g_error_free (error);
		return 0;
	}

	return pixbuf;
} 


gint
girda_load_gfx( Irda_applet_t *wdata )
{
	GError *error;
	int i;

	wdata->image_left = gtk_image_new ();
	wdata->image_middle = gtk_image_new ();
	wdata->image_right = gtk_image_new ();

	wdata->gfx_left[DISABLED] = 0;
	wdata->gfx_middle[DISABLED] = 0;
	wdata->gfx_right[DISABLED] = girda_load_pixbuf( SHARE_DIR"/pixmaps/girda_applet/irda_l_x.png");
	
	wdata->gfx_left[ENABLED] = 0;
	wdata->gfx_middle[ENABLED] = 0;
	wdata->gfx_right[ENABLED] = 0;

	wdata->gfx_middle[BLINK_ON] = 0;
	wdata->gfx_middle[BLINK_ON] = 0;
	wdata->gfx_right[BLINK_ON] = girda_load_pixbuf( SHARE_DIR"/pixmaps/girda_applet/irda_l_e.png");

	wdata->gfx_left[BLINK_OFF] = 0; 
	wdata->gfx_middle[BLINK_OFF] = 0; 
	wdata->gfx_right[BLINK_OFF] = girda_load_pixbuf( SHARE_DIR"/pixmaps/girda_applet/irda_l_d.png");

	wdata->gfx_left[DISCOVERED] = 0;
	wdata->gfx_middle[DISCOVERED] = 0;
	wdata->gfx_right[DISCOVERED] = girda_load_pixbuf( SHARE_DIR"/pixmaps/girda_applet/irda_l_e.png");

	gtk_image_set_from_pixbuf (GTK_IMAGE (wdata->image_left), wdata->gfx_left[DISABLED]);
	gtk_image_set_from_pixbuf (GTK_IMAGE (wdata->image_middle), wdata->gfx_middle[DISABLED]);
	gtk_image_set_from_pixbuf (GTK_IMAGE (wdata->image_right), wdata->gfx_right[DISABLED]);

	gtk_box_pack_start_defaults (GTK_BOX(wdata->box), wdata->image_left );
	gtk_box_pack_start_defaults (GTK_BOX(wdata->box), wdata->image_middle );
	gtk_box_pack_start_defaults (GTK_BOX(wdata->box), wdata->image_right );

	return FALSE;
}

gint
girda_create_layout( Irda_applet_t *wdata )
{

	wdata->eventstatus = gtk_event_box_new ();
	gtk_widget_show (wdata->eventstatus);
	gtk_container_add (GTK_CONTAINER (wdata->applet), 
			   wdata->eventstatus);
   
	wdata->box = gtk_hbox_new( FALSE, 2 );
	gtk_widget_show( wdata->box );
        gtk_container_add (GTK_CONTAINER (wdata->eventstatus), wdata->box);


	wdata->tips = gtk_tooltips_new();
	
	gtk_tooltips_set_tip( GTK_TOOLTIPS (wdata->tips), wdata->eventstatus,
			      _("No device detected!"),
			      NULL);

	girda_load_gfx( wdata );

	return FALSE;
}


static gboolean
girda_applet_fill (PanelApplet *applet)
{
	Irda_applet_t *wdata;

	wdata = g_new0( Irda_applet_t, 1 );

	wdata->applet = applet;
	
	girda_create_layout( wdata );

	wdata->status_window = 0;
	wdata->info_window = 0;
	wdata->number_detected = -1;
	wdata->detected_devices = g_strdup( _("No device detected!") );

	gnome_window_icon_set_default_from_file (SHARE_DIR"/pixmaps/girda_applet/irda-48.png");
	panel_applet_add_preferences (applet, "/schemas/apps/GIrDA_Applet/prefs", NULL);
	panel_applet_setup_menu_from_file (PANEL_APPLET (applet), 
					   NULL,
					   SHARE_DIR"/gnome-2.0/ui/GNOME_GIrDA_Applet.xml",
					   NULL,
					   GIrDA_Applet_menu_verbs,
					   wdata);

        g_signal_connect ( wdata->applet,
                          "button-release-event",
			   (GCallback) girda_rel_but_event_cb,
			   wdata);

	g_signal_connect( wdata->applet,
			  "change_orient",
			  (GCallback) girda_change_orient_cb, 
			  wdata);

	wdata->search_timeout = gtk_timeout_add( 2000, girda_search_cb, wdata );
	wdata->animate_timeout = gtk_timeout_add( 600, girda_animate_cb, wdata );
	
	girda_change_orient_cb( applet, 
				panel_applet_get_orient( applet ),
				wdata);
	



	gtk_widget_show_all( GTK_WIDGET(applet) );
	return TRUE;
}


static gboolean
girda_applet_factory (PanelApplet *applet,
		      const gchar          *iid,
		      gpointer              data)
{
	gboolean retval = FALSE;

	if (!strcmp (iid, "OAFIID:GNOME_GIrDA_Applet"))
		retval = girda_applet_fill (applet);
  
	return retval;
}


PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_GIrDA_Applet_Factory",
                             PANEL_TYPE_APPLET,
                             "girda_applet",
                             "0",
                             girda_applet_factory,
                             NULL)
      
