/*
 *  GNOME IrDA Monitor (girda)
 *  Copyright (C) 2001 Ragnar Henriksen <ragnar@juniks.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>

#include "girda_app.h"
#include "girda_applet.h"
#include "irda_misc.h"
#include "girda_cb.h"


void
girda_menu_activate_cb( GtkWidget *menu_item, PanelApplet *applet)
{
	g_print("girda_menu_activate_cb\n");
	g_print("Name of widget: %s\n", gtk_widget_get_name(menu_item));
}

gboolean  
girda_rel_but_event_cb(PanelApplet *applet, GdkEventButton *event, Irda_applet_t *data)
{
	if ( data && data->number_detected == -1 )
		return FALSE;

	if ( event->button == 1 ) {
		show_device_menu_cb( data );
		return TRUE;
	}

	return FALSE;
}

static void
on_dialog_response (GtkDialog *dialog, gint response, gpointer ok_handler_data)
{
	if (response == GTK_RESPONSE_CLOSE) {
		gtk_widget_destroy (GTK_WIDGET (dialog));
		dialog = 0;
	}
}


void
prop_cb( PanelApplet *applet, gpointer data )
{
	// FIXME
}

void
help_cb( PanelApplet *applet, gpointer data )
{
	GError *err;

	err = NULL;  
	gnome_help_display ("girda_applet", NULL, &err);

	if (err != NULL) {
		g_print ("%s \n", err->message);
                g_error_free (err);
                err = NULL;
	}
}


void
about_cb( PanelApplet *applet, gpointer data )
{
	GtkWidget   *about_box;
	GdkPixbuf   *pixbuf;
	GError      *error = NULL;
	gchar       *file;
   
	const gchar *authors[] = {
		"Ragnar Henriksen <ragnar@juniks.org>",
		NULL
	};

	const gchar *documenters[] = {
		NULL
	};

	const gchar *translator_credits = 0; //_("translator_credits");
   
	pixbuf = girda_load_pixbuf( SHARE_DIR"/pixmaps/girda_applet/irda-48.png");
   
	about_box = gnome_about_new (
		"GNOME IrDA Monitor", 
		VERSION,
		"Copyright (C) 2002  Ragnar Henriksen",
		_("This utility shows your IrDA connections."),
		authors,
		documenters,
		!strcmp (translator_credits, "translator_credits") ? translator_credits : NULL,
		pixbuf);
   
	if (pixbuf) 
		gdk_pixbuf_unref (pixbuf);
   
	gtk_window_set_wmclass (GTK_WINDOW (about_box), "GNOME IrDA monitor", "GNOME IrDA monitor");
	gtk_widget_show (about_box);

}

gint 
girda_search_cb( gpointer data )
{

        Irda_applet_t *wdata = (Irda_applet_t *)data;

        gint num;
        gchar *tooltip_postfix = irda_discover_device( &num );

	if ( num == -1 ) {
		g_free( tooltip_postfix );
		wdata->detected_devices = g_strdup(_("IrDA not enabled"));
		wdata->number_detected = num;
		gtk_tooltips_set_tip( GTK_TOOLTIPS (wdata->tips), wdata->eventstatus,
				      wdata->detected_devices,
				      NULL);
		return 1;
	}

        if ( (num != wdata->number_detected) ) {
                gchar *tooltip_prefix = {_("Devices found")};
                gchar *tooltip;
                wdata->number_detected = num;

                if ( !num )
                        tooltip = g_strdup(_("No device detected!"));
                else
                        tooltip = g_strdup_printf(_("%s:\n%s"), tooltip_prefix, tooltip_postfix);
                
                g_free(wdata->detected_devices);

                wdata->detected_devices = g_strdup( tooltip );

                g_free(tooltip);
        }


	gtk_tooltips_set_tip( GTK_TOOLTIPS (wdata->tips), wdata->eventstatus,
			      wdata->detected_devices,
			      NULL);

        g_free(tooltip_postfix);
        return 1;
}


gint 
girda_animate_cb( gpointer data )
{
	static gint last = 0;
	static gint updated = 0;

	Irda_applet_t *wdata = (Irda_applet_t *)data;

	if ( wdata->number_detected == -1 ) {
		gtk_image_set_from_pixbuf (GTK_IMAGE (wdata->image_right), 
					   wdata->gfx_left[DISABLED]);
		gtk_image_set_from_pixbuf (GTK_IMAGE (wdata->image_right), 
					   wdata->gfx_middle[DISABLED]);
		gtk_image_set_from_pixbuf (GTK_IMAGE (wdata->image_right), 
					   wdata->gfx_right[DISABLED]);
	} else if (wdata->number_detected == 0) {
                if ( last ) {
                        last = 0;
			gtk_image_set_from_pixbuf (GTK_IMAGE (wdata->image_right), 
						   wdata->gfx_right[BLINK_ON]);
                } else {
                        last = 1;
			gtk_image_set_from_pixbuf (GTK_IMAGE (wdata->image_right), 
						   wdata->gfx_right[BLINK_OFF]);
                }
		updated = 0;
        } else {
                if ( !updated ) {
			updated = 1;
			gtk_image_set_from_pixbuf (GTK_IMAGE (wdata->image_right), 
						   wdata->gfx_right[BLINK_ON]);
                }
        }

	return 1;
}


void
tooltip_show_cb( gpointer data ) 
{
        Irda_applet_t *wdata = (Irda_applet_t *)data;
        GtkWidget *frame;
        GtkWidget *text;
        GtkRequisition  req;
        gint width;
	gint height;
	gint x;
        gint y;

        if (! wdata->status_window ) {
                wdata->status_window = gtk_window_new (GTK_WINDOW_POPUP);

                frame = gtk_frame_new (NULL);
                gtk_widget_ref (frame);
                gtk_object_set_data_full (GTK_OBJECT (wdata->status_window), "frame", frame,
                                          (GtkDestroyNotify) gtk_widget_unref);
 
                gtk_widget_show (frame);
                gtk_container_add (GTK_CONTAINER (wdata->status_window), frame);
                gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);


                text = gtk_label_new (wdata->detected_devices);
                gtk_widget_show (text);
                gtk_container_add (GTK_CONTAINER (frame), text);

		/*
		 * Position the popup right next to the button.
		 */
		gtk_widget_size_request (wdata->status_window, &req);

       
		gdk_window_get_origin (wdata->image_right->window, &x, &y);
		gdk_drawable_get_size (wdata->image_right->window, &width, &height);
        
		switch ( wdata->orientation ) {
		case PANEL_APPLET_ORIENT_DOWN:
			x += (width - req.width) / 2;
			y += height;
			break;
			
		case PANEL_APPLET_ORIENT_UP:
			x += (width - req.width) / 2;
			y -= req.height;
			break;
			
		case PANEL_APPLET_ORIENT_LEFT:
			x -= req.width;
			y += (height - req.height) / 2;                 
			break;
			
		case PANEL_APPLET_ORIENT_RIGHT:
			x += width;
			y += (height - req.height) / 2;                 
			break;
			
		default:
			break;
		}
        
		if (x < 0) {
			x = 0;
		}
		if (y < 0) {
			y = 0;
		}
		
		gtk_window_move (GTK_WINDOW (wdata->status_window), x, y);
               
                gtk_widget_ref( wdata->status_window) ;
                gtk_widget_show(wdata->status_window);

                gtk_timeout_add( 3000, tooltip_hide_cb, wdata);
        }
}

gint
tooltip_hide_cb (gpointer data)
{
        Irda_applet_t *wdata = (Irda_applet_t *)data;
	
        if ( GTK_IS_WIDGET(wdata->status_window) ) {
                gtk_widget_destroy(wdata->status_window);
                wdata->status_window = 0;
        }
	
        return 0;
}

void
girda_change_orient_cb (PanelApplet *applet, PanelAppletOrient o, Irda_applet_t *data)
{
        tooltip_hide_cb (data);
        data->orientation = o;
}



void 
show_device_menu_cb( Irda_applet_t *data )
{
	GtkWidget *device_item;

	data->popup_menu = gtk_menu_new();
	gtk_menu_set_title( GTK_MENU(data->popup_menu), _("Popup Menu"));


	device_item = gtk_menu_item_new_with_mnemonic(_("_Info"));
	gtk_widget_set_name(device_item, "info");
	gtk_widget_show(device_item);
	gtk_menu_append( GTK_MENU(data->popup_menu), device_item );
	g_signal_connect (G_OBJECT (device_item),
			  "activate",
			  G_CALLBACK(show_device_info_cb), data);

	device_item = gtk_separator_menu_item_new();
	gtk_widget_show(device_item);
	gtk_menu_append( GTK_MENU(data->popup_menu), device_item );

	device_item = gtk_menu_item_new_with_mnemonic(_("_Send file..."));
	gtk_widget_set_name(device_item, "send");
	gtk_widget_show(device_item);
	gtk_menu_append( GTK_MENU(data->popup_menu), device_item );
	g_signal_connect (G_OBJECT (device_item),
			  "activate",
			  G_CALLBACK(girda_menu_activate_cb), data);

	device_item = gtk_menu_item_new_with_mnemonic(_("_Recieve file..."));
	gtk_widget_set_name(device_item, "recieve");
	gtk_widget_show(device_item);
	gtk_menu_append( GTK_MENU(data->popup_menu), device_item );
	g_signal_connect (G_OBJECT (device_item),
			  "activate",
			  G_CALLBACK(girda_menu_activate_cb), data);


	gtk_menu_popup( GTK_MENU(data->popup_menu), 0, 0, 0, data,
			0, gtk_get_current_event_time ());
	
	return;

}

GtkWidget *
show_device_info_cb( Irda_applet_t *data )
{
	GtkWidget *info;
	GtkWidget *vbox1;
	GtkWidget *close;
	GtkWidget *table1;
	GtkWidget *label1;
	GtkWidget *ck1;
	GtkWidget *aa1;
	gint i;
	gint y;
	gint numdev;
	struct _dev_info *devices;
	struct _dev_info *p;
	gchar *labels[] = {_("Hints"),
			   _("PNP"),
			   _("PDA"),
			   _("COMPUTER"),
			   _("PRINTER"),
			   _("MODEM"),
			   _("FAX"),
			   _("LAN"),
			   _("EXTENSION"),
			   _("TELEPHONY"),
			   _("FILESERVER"),
			   _("COMM"),
			   _("MESSAGE"),
			   _("HTTP"),
			   _("OBEX"),
			   NULL
	};

	devices = irda_get_hints( &numdev );
	if ( numdev <= 0 )
		return 0;

	info = gtk_dialog_new ();
	data->info_window = info;
	gtk_window_set_title (GTK_WINDOW (info), _("info"));
	gtk_window_set_resizable (GTK_WINDOW (info), FALSE);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (info), TRUE);
	gtk_dialog_set_default_response (GTK_DIALOG (info), GTK_RESPONSE_CLOSE);
 


	vbox1 =  GTK_DIALOG (info)->vbox;
	gtk_widget_show (vbox1);
	
	table1 = gtk_table_new (15, numdev+1, TRUE);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);

	/* Create labels */
	i = 0;
	while ( labels[i] ) {
		label1 = gtk_label_new (_(labels[i]));
		gtk_widget_show (label1);
		gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, i, i+1,
				  (GtkAttachOptions) (GTK_FILL),
				  (GtkAttachOptions) (0), 5, 0);
		gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
		gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
		i++;
	}

	/* Adding checkboxes */
	p = devices;
	i = 0;
	while ( p ) {
		label1 = gtk_label_new( p->name );
		gtk_widget_show (label1);
		gtk_table_attach (GTK_TABLE (table1), label1, i+1, i+2, 0, 1,
				  (GtkAttachOptions) (GTK_FILL),
				  (GtkAttachOptions) (0), 5, 0);
		gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
		gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
		
		y = 1;
		while ( labels[y] ) {
			ck1 = gtk_check_button_new();
			GTK_WIDGET_UNSET_FLAGS (ck1, GTK_CAN_FOCUS);
			gtk_widget_set_sensitive (ck1, FALSE);

			if ( (y <= 8) && (p->hints[0] & 1<<(y-1)) )
				gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(ck1), 1 );

			if ( (y > 7) && (p->hints[1] & 1<<(y-9)) )
				gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(ck1), 1 );

			gtk_widget_show (ck1);
			gtk_table_attach (GTK_TABLE (table1), ck1, i+1, i+2, y, y+1,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 5, 0);
			y++;
		}
		p = p->next;
		i++;
	}

	p = devices;
	while ( p ) {
		devices = p->next;
		free( p );
		p = devices;
	}

	/* Create close button */
	aa1 = GTK_DIALOG (info)->action_area;
	gtk_widget_show (aa1);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (aa1), GTK_BUTTONBOX_END);
	
	close = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (close);
	gtk_dialog_add_action_widget (GTK_DIALOG (info), close, GTK_RESPONSE_CLOSE);
	GTK_WIDGET_SET_FLAGS (close, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (info), "response",
			    GTK_SIGNAL_FUNC (on_dialog_response),
			    NULL);

	gtk_widget_show(info);
	
	return info;
}
