/*
 *  GNOME IrDA Monitor (girda)
 *  Copyright (C) 2001 Ragnar Henriksen <ragnar@juniks.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include <sys/socket.h>
#include <linux/types.h>
#include <linux/irda.h>

#ifdef ENABLE_OPENOBEX
#  include <openobex/obex.h>
#endif

#include "girda_app.h"
#include "irda_misc.h"

#ifdef ENABLE_OPENOBEX
void obex_event(obex_t *handle, obex_object_t *object, gint mode, gint event, gint obex_cmd, gint obex_rsp);
#endif

volatile int finished = FALSE;

gchar * 
irda_discover_device(gint *num)
{

        struct irda_device_list *list;
        struct irda_device_info *dev;
	gchar          *buf;
	gchar          *devices;
	gchar          *sep = _(", ");
        gint           s;
	gint           i;

	gint fd;
	gint len;

        fd = socket(AF_IRDA, SOCK_STREAM, 0);
	if ( fd == -1 ) {
		devices = 0;
		*num = -1;
		return devices;
	}
     
        len = sizeof(*list) + sizeof(*dev) * 10;
        buf = g_new0(gchar, len);

        list = (struct irda_device_list *)buf;

        dev = list->dev;
	s = len;
	devices = 0;

	if ( getsockopt(fd, SOL_IRLMP, IRLMP_ENUMDEVICES, buf, &s) == 0 ) {
		*num = list->len;
		for (i = 0; i < list->len ; i++) {
			devices = g_strjoin(sep, dev[i].info, devices, NULL);
		}
	}
	
        g_free(buf);
        close(fd);

        return devices;
}

struct _dev_info *
irda_get_hints( gint *num )
{

        struct irda_device_list *list;
        struct irda_device_info *dev;
	struct _dev_info *p;
	struct _dev_info *inf;
	gchar          *buf;
	gchar          *sep = _(", ");
        gint           s;
	gint           i;

	gint fd;
	gint len;

        fd = socket(AF_IRDA, SOCK_STREAM, 0);
	if ( fd == -1 ) {
		inf = 0;
		*num = -1;
		return inf;
	}
     
        len = sizeof(*list) + sizeof(*dev) * 10;
        buf = g_new0(gchar, len);

        list = (struct irda_device_list *)buf;

        dev = list->dev;
	s = len;
	inf = 0;
	if ( getsockopt(fd, SOL_IRLMP, IRLMP_ENUMDEVICES, buf, &s) == 0 ) {
		*num = list->len;
		for (i = 0; i < list->len ; i++) {
			p = (struct _dev_info *)malloc(sizeof(struct _dev_info));
			p->name = g_strdup(dev[i].info);
			p->hints[0] = dev[i].hints[0];
			p->hints[1] = dev[i].hints[1];
			p->next = inf;
			inf = p;
		}
	}
	
        g_free(buf);
        close(fd);

	return inf;
}


#ifdef ENABLE_OPENOBEX
gint 
irda_obex_init( obex_t *handle )
{
	g_print("init obex\n");
	handle = OBEX_Init(OBEX_TRANS_IRDA, obex_event, 0);

	return -1;
}
#endif

#ifdef ENABLE_OPENOBEX
gint 
irda_obex_send_file( obex_t *handle )
{
#if 0
	int ret;

	/* Try to connect to peer */
	ret = IrOBEX_TransportConnect(handle, "OBEX");
	if (ret < 0) {
		g_printf("Sorry, unable to connect!\n");
		return ret;
	}

	object = OBEX_ObjectNew(handle, OBEX_CMD_CONNECT);
	ret = do_sync_request(handle, object, FALSE);
	
	if( (object = build_object_from_file(handle, argv[1])) )        {
		ret = do_sync_request(handle, object, FALSE);
	}
	else    {
		perror("PUT failed");
	}

	object = OBEX_ObjectNew(handle, OBEX_CMD_DISCONNECT);
	ret = do_sync_request(handle, object, FALSE);

	printf("PUT successful\n");
#endif
	return -1;
}
#endif

#ifdef ENABLE_OPENOBEX
gint 
irda_obex_recieve_file( obex_t *handle )
{
#if 0
	g_print("Waiting for files\n");
	OBEX_ServerRegister(handle, "OBEX");

	while (!finished)
		OBEX_HandleInput(handle, 1);

	
#endif
	return -1;
}
#endif


#ifdef ENABLE_OPENOBEX
void 
obex_event(obex_t *handle, obex_object_t *object, gint mode, gint event, gint obex_cmd, gint obex_rsp)
{
	g_print("obex_event\n");
}
#endif
